<?php

/*
Plugin Name: EBN Clone Plugin
Description: EBN Clone Plugin
Version: 2.3.0
Plugin URI: https://app.easyblognetworks.com
*/


if ( version_compare( PHP_VERSION, '5.3.0', '<' ) ) {
	?>
	<div id="error-page">
		<p>This plugin requires PHP 5.3.0 or higher. Please contact your hosting provider about upgrading your
			server software. Your PHP version is <b>
				<?php
				echo PHP_VERSION;
				?>
			</b></p>
	</div>
	<?php
	die();
}

class EBNClone {
	const PluginHook          = 'ebnclone_installPlugin';
	const ThemeHook           = 'ebnclone_installTheme';
	const UninstallHook       = 'ebnclone_uninstallSelf';
	const EBNImportMutex      = 'ebnclone_lock';
	private $plugins_imported = false;

	function __construct() {
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
		add_action( self::PluginHook, array( $this, 'installPlugin' ) );
		add_action( self::ThemeHook, array( $this, 'installTheme' ) );
		add_action( self::UninstallHook, array( $this, 'selfUninstall' ) );
		add_filter( 'http_headers_useragent', array( $this, 'userAgent' ), 10 );
	}

	function userAgent() {
		return 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2840.71 Safari/537.36';
	}

	function admin_menu() {
		add_management_page(
			'EBN Clone',
			'EBN Clone',
			'manage_options',
			'ebn_clone',
			array(
				$this,
				'import_handler',
			)
		);
	}

	function import_handler() {
		?>
		<style>
			.ebn-skip {
				color: rgb(236, 184, 71);
				font-size: 13px;
				font-style: normal;
			}

			.ebn-add {
				color: #45AD6C;
				font-size: 13px;
				font-style: normal;
			}

			.ebn-media-problem {
				color: red;
			}

			.ebn-upload-form {
				background: #fafafa;
				border: 1px solid #e5e5e5;
				padding: 20px;
				margin: 5px 0;
			}

			.ebn-install-help {
				color: #999;
				font-size: 15px;
				font-style: normal;
				margin: 0;
				padding: 5px 0 0;
				text-align: left
			}
		</style>
		<div class="wrap">
			<h2>Cloning Tool</h2>

			<p>If you have posts or comments in another system, WordPress can import those into this site.</p>

			<h3>Clone Import</h3>
			<?php
			$this->handleImport();
			?>
			<div class="upload-plugin" style="display:block!important">
				<form method="post" enctype="multipart/form-data" class="ebn-upload-form"
					action="<?php echo self_admin_url( 'tools.php?page=ebn_clone' ); ?>">
					<?php wp_nonce_field( 'ebn_clone_import' ); ?>
					<?php $this->wp_hidden_field( 'action', 'ebn_clone_import' ); ?>
					<p class="ebn-install-help">If you have database zip file, you can specify url to it here:</p>
					<input type="url" size="80" id="urldb" name="urldb" required />

					<p class="ebn-install-help">If you have media zip file, you can specify url to it here:</p>
					<input type="url" size="80" id="urlmedia" name="urlmedia" />

					<p class="ebn-install-help">If you have template zip file, you can specify url to it here:</p>
					<input type="url" size="80" id="template_url" name="template_url" />

					<p class="ebn-install-help">If you have stylesheet zip file, you can specify url to it here:</p>
					<input type="url" size="80" id="stylesheet_url" name="stylesheet_url" />

					<p class="ebn-install-help">If you have package zip file, you can specify url to it here:</p>
					<input type="url" size="80" id="package_url" name="package_url" />
					<p><input type="checkbox" name="activate">Don't activate plugins and themes</p>
					<p>
						<?php submit_button( __( 'Import Now' ), 'button', 'import-db-submit', false ); ?>
					</p>
				</form>
			</div>
			<h3>Clone Export</h3>
			<?php
			$this->handleCreate();
			?>
			<div class="upload-plugin" style="display:block!important">
				<form method="post" enctype="multipart/form-data" class="ebn-upload-form"
					action="<?php echo self_admin_url( 'tools.php?page=ebn_clone' ); ?>">
					<?php wp_nonce_field( 'ebn_clone_create' ); ?>
					<?php $this->wp_hidden_field( 'action', 'ebn_clone_create' ); ?>
					<?php if ( is_readable( $this->backupZIP( 'db' ) ) ) : ?>
						<p class="ebn-install-help" for="urldb_out">Link to db backup of this site is:</p>
						<input type="url" size="80" id="urldb_out" name="urldb_urldb"
							value="<?php echo $this->backupURL( 'db' ); ?>" />
					<?php endif; ?>
					<?php if ( is_readable( $this->backupZIP( 'media' ) ) ) : ?>
						<p class="ebn-install-help" for="urlmedia_out">Link to media backup of this site is:</p>
						<input type="url" size="80" id="urlmedia_out" name="urlmedia_out"
							value="<?php echo $this->backupURL( 'media' ); ?>" />
					<?php endif; ?>
					<?php if ( is_readable( $this->backupZIP( 'template' ) ) ) : ?>
						<p class="ebn-install-help" for="urltemplate_out">Link to template backup of this site is:</p>
						<input type="url" size="80" id="urltemplate_out" name="urltemplate_out"
							value="<?php echo $this->backupURL( 'template' ); ?>" />
					<?php endif; ?>
					<?php if ( is_readable( $this->backupZIP( 'stylesheet' ) ) ) : ?>
						<p class="ebn-install-help" for="urlstylesheet_out">Link to stylesheet backup of this site is:</p>
						<input type="url" size="80" id="urlstylesheet_out" name="urlstylesheet_out"
							value="<?php echo $this->backupURL( 'stylesheet' ); ?>" />
					<?php endif; ?>
					<p>
						<?php submit_button( __( 'Create Now' ), 'button', 'create-db-submit', false ); ?>
					</p>

				</form>
			</div>
		</div>
		<?php
	}

	private function handleImport() {
		if ( isset( $_POST['action'] ) && sanitize_text_field( $_POST['action'] ) === 'ebn_clone_import' ) {
			$dburl          = sanitize_text_field( $_POST['urldb'] );
			$mediaurl       = sanitize_text_field( $_POST['urlmedia'] );
			$package_url    = sanitize_text_field( $_POST['package_url'] );
			$template_url   = sanitize_text_field( $_POST['template_url'] );
			$stylesheet_url = sanitize_text_field( $_POST['stylesheet_url'] );
			$activate       = true;
			if ( isset( $_POST['activate'] ) ) {
				$activate = false;
			}
			if ( get_transient( self::EBNImportMutex ) === self::EBNImportMutex ) {
				echo '<p>Import in progress!Please wait 2 minutes before doing another import!</p>';
				die();
			} else {
				set_transient( self::EBNImportMutex, self::EBNImportMutex, 120 );
			}

			// Predefined content
			if ( isset( $_POST['package_url'] ) && filter_var( $package_url, FILTER_VALIDATE_URL ) ) {
				$this->importPackage( $package_url );
			}
			// A custom template and stylesheet
			if ( isset( $_POST['template_url'] ) && filter_var( $template_url, FILTER_VALIDATE_URL ) ) {
				$this->importPackage( $template_url );
			}
			if ( isset( $_POST['stylesheet_url'] ) && filter_var( $stylesheet_url, FILTER_VALIDATE_URL ) ) {
				$this->importPackage( $stylesheet_url );
			}
			if ( isset( $_POST['urlmedia'] ) && filter_var( $mediaurl, FILTER_VALIDATE_URL ) ) {
				$this->importMediaBackup( $mediaurl );
			}

			// Needs to be last
			if ( isset( $_POST['urldb'] ) && filter_var( $dburl, FILTER_VALIDATE_URL ) ) {
				$this->importDBBackup( $dburl, $activate );
			}

			echo '<p id="restore-success">Installed custom plugins or themes need to be activated manually!</p>';
			delete_transient( self::EBNImportMutex );
			die();
		}
	}

	private function handleCreate() {
		if ( ! class_exists( 'ZipArchive' ) ) {
			?>
			<div id="error-page">
				<p>This plugin requires ZipArchive support to enable exporting. Please contact your hosting provider about upgrading
					your
					server software.</b></p>
			</div>
			<?php
		} elseif ( isset( $_POST['action'] ) && sanitize_text_field( $_POST['action'] ) === 'ebn_clone_create' ) {
			$this->createDBBackup();
			$this->createMediaBackup();
			$template   = get_option( 'template' );
			$stylesheet = get_option( 'stylesheet' );
			$this->createThemeBackup( $template, 'template' );
			if ( $template !== $stylesheet ) {
				$this->createThemeBackup( $stylesheet, 'stylesheet' );
			}
			echo '<p id="backup-success">Backups created!</p>';
		}
	}


	function wp_hidden_field( $name = '_wpnonce', $action = -1, $echo = true ) {
		$name         = esc_attr( $name );
		$hidden_field = '<input type="hidden" id="' . $name . '" name="' . $name . '" value="' . $action . '" />';

		if ( $echo ) {
			echo $hidden_field;
		}

		return $hidden_field;
	}

	function activatePlugin() {
		if ( ! $this->isEBN() ) {
			update_option( 'ebn_native_zip', class_exists( 'ZipArchive' ) ? 'true' : 'false' );
			update_option( 'ebn_php_version', sprintf( '%s', phpversion() ) );
		}
		wp_schedule_single_event( time() + 7200, self::UninstallHook, array( realpath( __DIR__ ) ) );
	}

	private function getID() {
		return wp_hash( get_option( 'siteurl' ) );
	}

	private function backupZIP( $type = 'db' ) {
		return sprintf( '%s/%s_%s.zip', $this->getBasePath(), $this->getID(), $type );
	}

	private function backupURL( $type = 'db' ) {
		return sprintf( '%s/%s_%s.zip', $this->getBaseURL(), $this->getID(), $type );
	}

	private function getBaseURL() {
		$wp = wp_upload_dir();
		return $wp['baseurl'];
	}

	private function getBasePath() {
		$wp = wp_upload_dir();
		return $wp['basedir'];
	}

	private function createDBBackup() {
		global $wpdb;
		$z   = new ZipArchive();
		$res = $z->open( $this->backupZIP(), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE );
		if ( $res !== true ) {
			echo "<p class='ebn-add'>Creating backups in ZIP files is not allowed on this system!</p>";
			die();
		}
		$all = array_merge( $wpdb->tables, $wpdb->global_tables );
		foreach ( $all as $table ) {
			if ( $table === 'usermeta' || $table === 'users' ) {
				continue;
			}
			echo "<p class='ebn-add'>Adding  " . $table . '.</p>';
			@set_time_limit( 60 );
			$table_pf = $wpdb->get_blog_prefix() . $table;
			if ( $table === 'options' ) {
				$db = $wpdb->get_results( "SELECT * FROM $table_pf WHERE option_name NOT LIKE '%transient%'", OBJECT );
			} else {
				$db = $wpdb->get_results( "SELECT * FROM $table_pf", OBJECT );
			}

			$len = count( $db );
			if ( ! $len ) {
				continue;
			}

			$pack = 0;
			$step = 4;
			if ( $len === 1 ) {
				$z->addFromString( $table . '_' . $pack . '.json', json_encode( $db ) );
			} elseif ( $len > 1 ) {
				if ( $step >= $len ) {
					$step = $len - 1;
				}
				foreach ( range( 0, $len - 1, $step ) as $start ) {
					$slice = array_slice( $db, $start, $step );
					$z->addFromString( $table . '_' . $pack . '.json', json_encode( $slice ) );
					++$pack;
				}
			}
		}
		$z->close();
		update_option( 'ebn_db', $this->backupURL( 'db' ) );
	}

	private function createMediaBackup() {
		$z   = new ZipArchive();
		$res = $z->open( $this->backupZIP( 'media' ), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE );
		if ( $res !== true ) {
			echo "<p class='ebn-add'>Creating backups in ZIP files is not allowed on this system!</p>";
			die();
		}
		$dir_iterator = new RecursiveDirectoryIterator( $this->getBasePath() );
		$iterator     = new RecursiveIteratorIterator( $dir_iterator );
		$close_zip    = false;
		$zip_size     = 0;
		foreach ( $iterator as $file ) {
			$close_zip = true;
			@set_time_limit( 60 );
			if ( $file->getSize() > 10485760 ) {
				echo "<p class='ebn-skip'>Skipping " . $file->getFilename() . ' file because it is larger than 10MB.</p>';
				continue;
			}
			if ( preg_match( '/\.(?:jpg|png|gif|jpeg)$/i', $file->getFilename() ) && $file->isFile() && $file->Isreadable() ) {
				$filename = str_replace( $this->getBasePath() . '/', '', $file->getRealPath() );
				echo "<p class='ebn-add'>Adding  " . $filename . '.</p>';
				$z->addFile( $file->getRealPath(), $filename );
				$zip_size += filesize( $file->getRealPath() );
			}
		}

		if ( $close_zip ) {
			$z->close();

			if ( $zip_size > 209715200 ) { // 209715200 === 200MB
				$human_size = size_format( $zip_size, 2 );
				echo "<p class='ebn-media-problem'>Media archive is too big ($human_size). The limit is 200MB.</p>";
			} else {
				update_option( 'ebn_media', $this->backupURL( 'media' ) );
			}
		} else {
			echo "<p class='ebn-skip'>No media backup created, no valid files found.</p>";
		}
	}

	private function createThemeBackup( $theme, $variant = 'template' ) {
		$z   = new ZipArchive();
		$res = $z->open( $this->backupZIP( $variant ), ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE );
		if ( $res !== true ) {
			echo "<p class='ebn-add'>Creating backups in ZIP files is not allowed on this system!</p>";
			die();
		}
		$theme_path   = join( array( WP_CONTENT_DIR, 'themes', $theme ), '/' );
		$dir_iterator = new RecursiveDirectoryIterator( $theme_path );
		$iterator     = new RecursiveIteratorIterator( $dir_iterator );
		$close_zip    = false;
		$zip_size     = 0;
		foreach ( $iterator as $file ) {
			$close_zip = true;
			if ( $file->isFile() && $file->Isreadable() ) {
				$filename = str_replace( WP_CONTENT_DIR . '/', '', $file->getRealPath() );
				echo "<p class='ebn-add'>Adding $variant  " . $filename . '.</p>';
				$z->addFile( $file->getRealPath(), $filename );
				$zip_size += filesize( $file->getRealPath() );
			}
		}

		if ( $close_zip ) {
			$z->close();
			if ( $zip_size > 209715200 ) { // 209715200 === 200MB
				$human_size = size_format( $zip_size, 2 );
				echo "<p class='ebn-$variant-problem'>$variant archive is too big ($human_size). The limit is 200MB.</p>";
			} else {
				update_option( "ebn_$variant", $this->backupURL( $variant ) );
			}
		} else {
			echo "<p class='ebn-skip'>No $variant backup created, no valid files found.</p>";
		}
	}

	private function importDBBackup( $dburl, $activate ) {
		global $wpdb;
		$up        = wp_upload_dir();
		$wasRemote = false;
		if ( strpos( $dburl, $up['url'] ) !== false ) {
			$file = str_replace( $up['url'], $up['path'], $dburl );
		} else {
			$file      = download_url( $dburl );
			$wasRemote = true;
			if ( is_wp_error( $file ) ) {
				wp_die( sprintf( '%s cannot be downloaded. Remote responded with %s', $dburl, var_export( $file->get_error_messages(), true ) ) );
			}
		}

		$target = sprintf( '%s/%s', $this->getBasePath(), 'db' );
		if ( defined( 'WP_TEMP_DIR' ) ) {
			$target = sprintf( '%s/%s', WP_TEMP_DIR, 'db' );
		}
		$this->wipeDir( $target );
		mkdir( $target );
		WP_Filesystem();
		$status = unzip_file( $file, $target );
		if ( is_wp_error( $status ) ) {
			wp_die( sprintf( '%s cannot be unzipped. Error %s', $dburl, $status ) );
		}
		if ( $wasRemote ) {
			@unlink( $file );
		}

		$this->maintenance_mode( true );
		$all = array_merge( $wpdb->tables, $wpdb->global_tables );
		$wpdb->query( "DELETE FROM $wpdb->postmeta" );
		$wpdb->query( "DELETE FROM $wpdb->posts" );
		$wpdb->query( "DELETE FROM $wpdb->terms" );
		$wpdb->query( "DELETE FROM $wpdb->term_relationships" );
		$wpdb->query( "DELETE FROM $wpdb->term_taxonomy" );
		$wpdb->query( "DELETE FROM $wpdb->comments" );
		$wpdb->query( "DELETE FROM $wpdb->commentmeta" );
		$wpdb->query( "DELETE FROM $wpdb->options WHERE `option_name` LIKE ('_site_transient%')" );
		$wpdb->query( "DELETE FROM $wpdb->options WHERE `option_name` LIKE ('_transient%')" );
		foreach ( $all as $table ) {
			@set_time_limit( 600 );
			$wpdb->query( 'BEGIN;' );
			$path  = sprintf( '%s/%s_*.json', $target, $table );
			$files = glob( $path );
			foreach ( $files as $f ) {
				if ( $table === 'usermeta' || $table === 'users' ) {
					continue;
				} elseif ( $table === 'options' ) {
					$this->importOptions( $f, $activate );
				} elseif ( $table === 'posts' ) {
					$this->importPosts( $f );
				} else {
					$this->importDb( $table, $f );
				}
				@unlink( $f );
			}
			$this->resetRoles();
			echo "<p class='ebn-add'>Imported  " . $table . '.</p>';
			$wpdb->query( 'COMMIT;' );
		}
		$this->maintenance_mode( false );
		$this->wipeDir( $target );
	}

	/**
	 * @param $file
	 */
	public function importPosts( $file ) {
		global $wpdb;
		if ( ! is_readable( $file ) ) {
			return;
		}
		$current_user = wp_get_current_user();

		$entries = json_decode( file_get_contents( $file ), true );
		$allowed = array(
			'ID',
			'post_author',
			'post_date',
			'post_date_gmt',
			'post_content',
			'post_title',
			'post_excerpt',
			'post_status',
			'comment_status',
			'ping_status',
			'post_password',
			'post_name',
			'to_ping',
			'pinged',
			'post_modified',
			'post_modified_gmt',
			'post_content_filtered',
			'post_parent',
			'guid',
			'menu_order',
			'post_type',
			'post_mime_type',
			'comment_count',
		);

		foreach ( $entries as $entry ) {
			$entry['post_author'] = $current_user->ID;
			$filtered             = array_filter(
				$entry,
				function ( $key ) use ( $allowed ) {
					return in_array( $key, $allowed );
				},
				ARRAY_FILTER_USE_KEY
			);

			if ( $wpdb->replace( $wpdb->posts, $filtered ) ) {
				echo "<p class='ebn-add'>Imported post " . $filtered['post_title'] . '.</p>';
			} else {
				echo "<p class='ebn-skip'>Failed importing post " . $filtered['post_title'] . '.</p>';
			}
		}
	}

	function resetRoles() {
		$roles = array(
			'editor'        => array(
				'name'         => 'Editor',
				'capabilities' => array(
					'moderate_comments'       => true,
					'manage_categories'       => true,
					'manage_links'            => true,
					'upload_files'            => true,
					'unfiltered_html'         => true,
					'edit_posts'              => true,
					'edit_others_posts'       => true,
					'edit_published_posts'    => true,
					'publish_posts'           => true,
					'edit_pages'              => true,
					'read'                    => true,
					'level_7'                 => true,
					'level_6'                 => true,
					'level_5'                 => true,
					'level_4'                 => true,
					'level_3'                 => true,
					'level_2'                 => true,
					'level_1'                 => true,
					'level_0'                 => true,
					'edit_others_pages'       => true,
					'edit_published_pages'    => true,
					'publish_pages'           => true,
					'delete_pages'            => true,
					'delete_others_pages'     => true,
					'delete_published_pages'  => true,
					'delete_posts'            => true,
					'delete_others_posts'     => true,
					'delete_published_posts'  => true,
					'delete_private_posts'    => true,
					'edit_private_posts'      => true,
					'read_private_posts'      => true,
					'delete_private_pages'    => true,
					'edit_private_pages'      => true,
					'read_private_pages'      => true,
					'wpseo_bulk_edit'         => true,
					'ef_view_calendar'        => true,
					'edit_post_subscriptions' => true,
					'ef_view_story_budget'    => true,
				),
			),
			'author'        => array(
				'name'         => 'Author',
				'capabilities' => array(
					'upload_files'            => true,
					'edit_posts'              => true,
					'edit_published_posts'    => true,
					'publish_posts'           => true,
					'read'                    => true,
					'level_2'                 => true,
					'level_1'                 => true,
					'level_0'                 => true,
					'delete_posts'            => true,
					'delete_published_posts'  => true,
					'ef_view_calendar'        => true,
					'edit_post_subscriptions' => true,
					'ef_view_story_budget'    => true,
				),
			),
			'contributor'   => array(
				'name'         => 'Contributor',
				'capabilities' => array(
					'edit_posts'           => true,
					'read'                 => true,
					'level_1'              => true,
					'level_0'              => true,
					'delete_posts'         => true,
					'ef_view_calendar'     => true,
					'ef_view_story_budget' => true,
				),
			),
			'subscriber'    => array(
				'name'         => 'Subscriber',
				'capabilities' => array(
					'read'    => true,
					'level_0' => true,
				),
			),
			'administrator' => array(
				'name'         => 'Administrator',
				'capabilities' => array(
					'switch_themes'          => 1,
					'edit_themes'            => 1,
					'activate_plugins'       => 1,
					'edit_plugins'           => 1,
					'edit_users'             => 1,
					'edit_files'             => 1,
					'manage_options'         => 1,
					'moderate_comments'      => 1,
					'manage_categories'      => 1,
					'manage_links'           => 1,
					'upload_files'           => 1,
					'import'                 => 1,
					'unfiltered_html'        => 1,
					'edit_posts'             => 1,
					'edit_others_posts'      => 1,
					'edit_published_posts'   => 1,
					'publish_posts'          => 1,
					'edit_pages'             => 1,
					'read'                   => 1,
					'level_10'               => 1,
					'level_9'                => 1,
					'level_8'                => 1,
					'level_7'                => 1,
					'level_6'                => 1,
					'level_5'                => 1,
					'level_4'                => 1,
					'level_3'                => 1,
					'level_2'                => 1,
					'level_1'                => 1,
					'level_0'                => 1,
					'edit_others_pages'      => 1,
					'edit_published_pages'   => 1,
					'publish_pages'          => 1,
					'delete_pages'           => 1,
					'delete_others_pages'    => 1,
					'delete_published_pages' => 1,
					'delete_posts'           => 1,
					'delete_others_posts'    => 1,
					'delete_published_posts' => 1,
					'delete_private_posts'   => 1,
					'edit_private_posts'     => 1,
					'read_private_posts'     => 1,
					'delete_private_pages'   => 1,
					'edit_private_pages'     => 1,
					'read_private_pages'     => 1,
					'delete_users'           => 1,
					'create_users'           => 1,
					'unfiltered_upload'      => 1,
					'edit_dashboard'         => 1,
					'update_plugins'         => 1,
					'delete_plugins'         => 1,
					'install_plugins'        => 1,
					'update_themes'          => 1,
					'install_themes'         => 1,
					'update_core'            => 1,
					'list_users'             => 1,
					'remove_users'           => 1,
					'add_users'              => 1,
					'promote_users'          => 1,
					'edit_theme_options'     => 1,
					'delete_themes'          => 1,
					'export'                 => 1,
					'wpseo_manage_options'   => true,
				),
			),
		);

		update_option( 'wp_user_roles', $roles );
		$users = get_users( array( 'role' => 'administrator' ) );
		if ( count( $users ) > 0 ) {
			foreach ( $users as $user ) {
				$wp_user_object = new WP_User( $user->ID );
				$wp_user_object->set_role( 'administrator' );
			}
		}
	}


	private function importOptions( $file, $activate ) {
		global $wpdb;
		$options = json_decode( file_get_contents( $file ) );
		foreach ( $options as $option ) {
			$name = $option->option_name;
			if (
				$name === 'siteurl' ||
				$name === 'home' ||
				$name === 'cron' ||
				$name === 'upload_path' ||
				$name === 'recently_activated' ||
				$name === 'auth_key' ||
				$name === 'auth_salt' ||
				$name === 'logged_in_key' ||
				$name === 'logged_in_salt' ||
				$name === 'cron' ||
				$name === 'initial_db_version' ||
				$name === 'db_version' ||
				stristr( $name, '_transient' ) !== false ||
				stristr( $name, '_site_transient' ) !== false
			) {
				continue;
			}
			if ( $name === 'active_plugins' && ! $this->plugins_imported ) {
				$plugins = (array) maybe_unserialize( $option->option_value );
				foreach ( $plugins as $plugin ) {
					@set_time_limit( 60 );
					if ( ! is_plugin_active( $plugin ) ) {
						$this->installPlugin( $plugin );
						echo "<p class='ebn-add'>Installed plugin " . $plugin . '.</p>';
					}
				}
				$this->plugins_imported = true;
				if ( $activate ) {
					require_once ABSPATH . 'wp-admin/includes/plugin.php';
					activate_plugins( array_keys( get_plugins() ), '', true, true );
				}

				continue;
			}

			echo "<p class='ebn-add'>Set " . $name . '.</p>';
			update_option( $name, maybe_unserialize( $option->option_value ), $option->autoload );
		}
	}

	public function maintenance_mode( $enable = false ) {
		global $wp_filesystem;
		$file = $wp_filesystem->abspath() . '.maintenance';
		if ( $enable ) {
			// Create maintenance file to signal that we are upgrading
			$maintenance_string = '<?php $upgrading = ' . time() . '; ?>';
			$wp_filesystem->delete( $file );
			$wp_filesystem->put_contents( $file, $maintenance_string, FS_CHMOD_FILE );
		} elseif ( ! $enable && $wp_filesystem->exists( $file ) ) {
			$wp_filesystem->delete( $file );
		}
	}

	private function importDb( $table, $file ) {
		global $wpdb;
		if ( ! is_readable( $file ) ) {
			return;
		}
		$entries = json_decode( file_get_contents( $file ), true );
		foreach ( $entries as $entry ) {
			$wpdb->replace( $wpdb->$table, $entry );
		}
	}

	private function importMediaBackup( $mediaurl ) {
		$file = download_url( $mediaurl );
		if ( is_wp_error( $file ) ) {
			wp_die( sprintf( '%s cannot be downloaded. Remote responded with %s', $mediaurl, var_export( $file->get_error_messages(), true ) ) );
		}
		WP_Filesystem();
		$status = unzip_file( $file, $this->getBasePath() );
		if ( is_wp_error( $status ) ) {
			wp_die( sprintf( '%s cannot be unzipped. Error: %s', $mediaurl, $status ) );
		}
		@unlink( $file );
	}

	public function installPlugin( $plugin ) {
		require_once ABSPATH . 'wp-admin/includes/file.php';
		$slug = dirname( plugin_basename( $plugin ) );
		if ( is_readable( WP_PLUGIN_DIR . '/' . $slug ) ) {
			return;
		}
		$file = download_url( sprintf( 'https://downloads.wordpress.org/plugin/%s.zip', $slug ) );
		if ( is_wp_error( $file ) ) {
			return;
		}
		WP_Filesystem();
		unzip_file( $file, WP_PLUGIN_DIR );
		@unlink( $file );
	}

	public function selfUninstall( $path ) {
		if ( stristr( $path, 'clone' ) !== false ) {
			$this->wipeDir( $path );
		}
	}

	public function installTheme( $theme ) {
		require ABSPATH . 'wp-admin/includes/file.php';

		if ( is_readable( get_theme_root() . '/' . $theme ) ) {
			return;
		}
	}

	private function wipeDir( $dir ) {
		if ( is_readable( $dir ) ) {
			$it    = new RecursiveDirectoryIterator( $dir, RecursiveDirectoryIterator::SKIP_DOTS );
			$files = new RecursiveIteratorIterator(
				$it,
				RecursiveIteratorIterator::CHILD_FIRST
			);
			foreach ( $files as $file ) {
				if ( $file->isDir() ) {
					@rmdir( $file->getRealPath() );
				} else {
					@unlink( $file->getRealPath() );
				}
			}
			@rmdir( $dir );
		}
	}

	function isEBN() {
		return defined( 'DB_VERSION_SECURE' );
	}

	private function importPackage( $package_url ) {
		$file = download_url( $package_url );
		if ( is_wp_error( $file ) ) {
			return;
		}
		WP_Filesystem();
		$status = unzip_file( $file, WP_CONTENT_DIR );
		if ( $status ) {
			echo "<p class='ebn-add'>Installed " . $package_url . '.</p>';
		}
		@unlink( $file );
	}
}

// Inside WordPress
if ( defined( 'ABSPATH' ) ) {
	$EBNCloneInstance = new EBNClone();
	register_activation_hook( __FILE__, array( &$EBNCloneInstance, 'activatePlugin' ) );
}
